@@ -1,14 +1,26 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2013-2014, 2016 Google, Inc.
-# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Holger Peters <email@holger-peters.de>
+# Copyright (c) 2009 James Lingard <jchl@aristanetworks.com>
+# Copyright (c) 2012-2014 Google, Inc.
+# Copyright (c) 2014-2017 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 David Shea <dshea@redhat.com>
-# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
-# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
+# Copyright (c) 2014 Steven Myint <hg@stevenmyint.com>
+# Copyright (c) 2014 Holger Peters <email@holger-peters.de>
+# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
+# Copyright (c) 2015 Anentropic <ego@anentropic.com>
 # Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
+# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016 Alexander Todorov <atodorov@otb.bg>
+# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Jürgen Hermann <jh@web.de>
+# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2016 Filipe Brandenburger <filbranden@google.com>
+# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
+# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -25,18 +37,19 @@ import re
 import shlex
 import sys
 
-import editdistance
 import six
 
 import astroid
 import astroid.context
 import astroid.arguments
-from astroid import exceptions
+import astroid.nodes
+from astroid import exceptions, decorators
 from astroid.interpreter import dunder_lookup
 from astroid import objects
 from astroid import bases
+from astroid import modutils
 
-from pylint.interfaces import IAstroidChecker, INFERENCE, INFERENCE_FAILURE
+from pylint.interfaces import IAstroidChecker, INFERENCE
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import (
     is_super, check_messages, decorated_with_property,
@@ -50,10 +63,12 @@ from pylint.checkers.utils import (
     has_known_bases,
     is_builtin_object,
     singledispatch)
-
+from pylint.utils import get_global_option
 
 BUILTINS = six.moves.builtins.__name__
-STR_FORMAT = "%s.str.format" % BUILTINS
+STR_FORMAT = {"%s.str.format" % BUILTINS}
+if six.PY2:
+    STR_FORMAT.add("%s.unicode.format" % BUILTINS)
 
 
 def _unflatten(iterable):
@@ -118,6 +133,25 @@ def _(node):
     return itertools.chain(values, other_values)
 
 
+def _string_distance(seq1, seq2):
+    seq2_length = len(seq2)
+
+    row = list(range(1, seq2_length + 1)) + [0]
+    for seq1_index, seq1_char in enumerate(seq1):
+        last_row = row
+        row = [0] * seq2_length + [seq1_index + 1]
+
+        for seq2_index, seq2_char in enumerate(seq2):
+            row[seq2_index] = min(
+                last_row[seq2_index] + 1,
+                row[seq2_index - 1] + 1,
+                last_row[seq2_index - 1] + (seq1_char != seq2_char)
+
+            )
+
+    return row[seq2_length - 1]
+
+
 def _similar_names(owner, attrname, distance_threshold, max_choices):
     """Given an owner and a name, try to find similar names
 
@@ -131,7 +165,7 @@ def _similar_names(owner, attrname, distance_threshold, max_choices):
         if name == attrname:
             continue
 
-        distance = editdistance.eval(attrname, name)
+        distance = _string_distance(attrname, name)
         if distance <= distance_threshold:
             possible_names.append((name, distance))
 
@@ -163,6 +197,13 @@ MSGS = {
               'no-member',
               'Used when a variable is accessed for an unexistent member.',
               {'old_names': [('E1103', 'maybe-no-member')]}),
+    'I1101': ('%s %r has not %r member%s, but source is unavailable. Consider '
+              'adding this module to extension-pkg-whitelist if you want '
+              'to perform analysis based on run-time introspection of living objects.',
+              'c-extension-no-member',
+              'Used when a variable is accessed for non-existent member of C '
+              'extension. Due to unavailability of source static analysis is impossible, '
+              'but it may be performed by introspecting living objects in run-time.'),
     'E1102': ('%s is not callable',
               'not-callable',
               'Used when an object being called has been inferred to a non \
@@ -242,11 +283,19 @@ MSGS = {
               'Emitted whenever we can detect that a class is using, '
               'as a metaclass, something which might be invalid for using as '
               'a metaclass.'),
+    'W1113': ('Keyword argument before variable positional arguments list '
+              'in the definition of %s function',
+              'keyword-arg-before-vararg',
+              'When defining a keyword argument before variable positional arguments, one can '
+              'end up in having multiple values passed for the aforementioned parameter in '
+              'case the method is called with keyword arguments.'),
     }
 
 # builtin sequence types in Python 2 and 3.
-SEQUENCE_TYPES = set(['str', 'unicode', 'list', 'tuple', 'bytearray',
-                      'xrange', 'range', 'bytes', 'memoryview'])
+SEQUENCE_TYPES = {
+    'str', 'unicode', 'list', 'tuple', 'bytearray',
+    'xrange', 'range', 'bytes', 'memoryview'
+}
 
 
 def _emit_no_member(node, owner, owner_name, ignored_mixins):
@@ -272,7 +321,7 @@ def _emit_no_member(node, owner, owner_name, ignored_mixins):
         return False
     if isinstance(owner, astroid.FunctionDef) and owner.decorators:
         return False
-    if isinstance(owner, astroid.Instance):
+    if isinstance(owner, (astroid.Instance, astroid.ClassDef)):
         if owner.has_dynamic_getattr() or not has_known_bases(owner):
             return False
     if isinstance(owner, objects.Super):
@@ -461,6 +510,10 @@ def _infer_from_metaclass_constructor(cls, func):
     return inferred or None
 
 
+def _is_c_extension(module_node):
+    return not modutils.is_standard_module(module_node.name) and not module_node.fully_defined()
+
+
 class TypeChecker(BaseChecker):
     """try to find bugs in the code using type inference
     """
@@ -473,7 +526,17 @@ class TypeChecker(BaseChecker):
     msgs = MSGS
     priority = -1
     # configuration options
-    options = (('ignore-mixin-members',
+    options = (('ignore-on-opaque-inference',
+                {'default': True, 'type': 'yn', 'metavar': '<y_or_n>',
+                 'help': 'This flag controls whether pylint should warn about '
+                         'no-member and similar checks whenever an opaque object '
+                         'is returned when inferring. The inference can return '
+                         'multiple potential results while evaluating a Python object, '
+                         'but some branches might not be evaluated, which results in '
+                         'partial inference. In that case, it might be useful to still emit '
+                         'no-member and other checks for the rest of the inferred objects.'}
+               ),
+               ('ignore-mixin-members',
                 {'default' : True, 'type' : 'yn', 'metavar': '<y_or_n>',
                  'help' : 'Tells whether missing members accessed in mixin \
 class should be ignored. A mixin class is detected if its name ends with \
@@ -546,6 +609,10 @@ accessed. Python regular expressions are accepted.'}
                ),
               )
 
+    @decorators.cachedproperty
+    def _suggestion_mode(self):
+        return get_global_option(self, 'suggestion-mode', default=True)
+
     def open(self):
         # do this in open since config not fully initialized in __init__
         # generated_members may contain regular expressions
@@ -555,9 +622,18 @@ accessed. Python regular expressions are accepted.'}
         if isinstance(self.config.generated_members, six.string_types):
             gen = shlex.shlex(self.config.generated_members)
             gen.whitespace += ','
-            gen.wordchars += '[]-+\.*?()|'
+            gen.wordchars += r'[]-+\.*?()|'
             self.config.generated_members = tuple(tok.strip('"') for tok in gen)
 
+    @check_messages('keyword-arg-before-vararg')
+    def visit_functiondef(self, node):
+        # check for keyword arg before varargs
+        if node.args.vararg and node.args.defaults:
+            self.add_message('keyword-arg-before-vararg', node=node,
+                             args=(node.name))
+
+    visit_asyncfunctiondef = visit_functiondef
+
     @check_messages('invalid-metaclass')
     def visit_classdef(self, node):
 
@@ -592,7 +668,7 @@ accessed. Python regular expressions are accepted.'}
     def visit_delattr(self, node):
         self.visit_attribute(node)
 
-    @check_messages('no-member')
+    @check_messages('no-member', 'c-extension-no-member')
     def visit_attribute(self, node):
         """check that the accessed attribute exists
 
@@ -609,18 +685,26 @@ accessed. Python regular expressions are accepted.'}
                 return
 
         try:
-            infered = list(node.expr.infer())
+            inferred = list(node.expr.infer())
         except exceptions.InferenceError:
             return
+
         # list of (node, nodename) which are missing the attribute
         missingattr = set()
-        inference_failure = False
-        for owner in infered:
-            # skip yes object
-            if owner is astroid.YES:
-                inference_failure = True
-                continue
 
+        non_opaque_inference_results = [
+            owner for owner in inferred
+            if owner is not astroid.Uninferable
+            and not isinstance(owner, astroid.nodes.Unknown)
+        ]
+        if (len(non_opaque_inference_results) != len(inferred)
+                and self.config.ignore_on_opaque_inference):
+            # There is an ambiguity in the inference. Since we can't
+            # make sure that we won't emit a false positive, we just stop
+            # whenever the inference returns an opaque inference object.
+            return
+
+        for owner in non_opaque_inference_results:
             name = getattr(owner, 'name', None)
             if _is_owner_ignored(owner, name, self.config.ignored_classes,
                                  self.config.ignored_modules):
@@ -644,6 +728,7 @@ accessed. Python regular expressions are accepted.'}
                 # So call this only after the call has been made.
                 if not _emit_no_member(node, owner, name,
                                        self.config.ignore_mixin_members):
+
                     continue
                 missingattr.add((owner, name))
                 continue
@@ -661,19 +746,27 @@ accessed. Python regular expressions are accepted.'}
                 if actual in done:
                     continue
                 done.add(actual)
-                confidence = INFERENCE if not inference_failure else INFERENCE_FAILURE
 
-                if self.config.missing_member_hint:
-                    hint = _missing_member_hint(owner, node.attrname,
-                                                self.config.missing_member_hint_distance,
-                                                self.config.missing_member_max_choices)
-                else:
-                    hint = ""
-
-                self.add_message('no-member', node=node,
+                msg, hint = self._get_nomember_msgid_hint(node, owner)
+                self.add_message(msg, node=node,
                                  args=(owner.display_type(), name,
                                        node.attrname, hint),
-                                 confidence=confidence)
+                                 confidence=INFERENCE)
+
+    def _get_nomember_msgid_hint(self, node, owner):
+        suggestions_are_possible = self._suggestion_mode and isinstance(owner, astroid.Module)
+        if suggestions_are_possible and _is_c_extension(owner):
+            msg = 'c-extension-no-member'
+            hint = ""
+        else:
+            msg = 'no-member'
+            if self.config.missing_member_hint:
+                hint = _missing_member_hint(owner, node.attrname,
+                                            self.config.missing_member_hint_distance,
+                                            self.config.missing_member_max_choices)
+            else:
+                hint = ""
+        return msg, hint
 
     @check_messages('assignment-from-no-return', 'assignment-from-none')
     def visit_assign(self, node):
@@ -703,16 +796,16 @@ accessed. Python regular expressions are accepted.'}
             else:
                 self.add_message('assignment-from-none', node=node)
 
-    def _check_uninferable_callfunc(self, node):
+    def _check_uninferable_call(self, node):
         """
-        Check that the given uninferable CallFunc node does not
+        Check that the given uninferable Call node does not
         call an actual function.
         """
         if not isinstance(node.func, astroid.Attribute):
             return
 
         # Look for properties. First, obtain
-        # the lhs of the Getattr node and search the attribute
+        # the lhs of the Attribute node and search the attribute
         # there. If that attribute is a property or a subclass of properties,
         # then most likely it's not callable.
 
@@ -776,7 +869,7 @@ accessed. Python regular expressions are accepted.'}
                 self.add_message('not-callable', node=node,
                                  args=node.func.as_string())
 
-        self._check_uninferable_callfunc(node)
+        self._check_uninferable_call(node)
 
         try:
             called, implicit_args, callable_name = _determine_callable(called)
@@ -862,7 +955,7 @@ accessed. Python regular expressions are accepted.'}
                     # by keyword argument, as in `.format(self=self)`.
                     # It's perfectly valid to so, so we're just skipping
                     # it if that's the case.
-                    if not (keyword == 'self' and called.qname() == STR_FORMAT):
+                    if not (keyword == 'self' and called.qname() in STR_FORMAT):
                         self.add_message('redundant-keyword-arg',
                                          node=node, args=(keyword, callable_name))
                 else:
@@ -921,15 +1014,15 @@ accessed. Python regular expressions are accepted.'}
     @check_messages('invalid-sequence-index')
     def visit_index(self, node):
         if not node.parent or not hasattr(node.parent, "value"):
-            return
+            return None
         # Look for index operations where the parent is a sequence type.
         # If the types can be determined, only allow indices to be int,
         # slice or instances with __index__.
         parent_type = safe_infer(node.parent.value)
         if not isinstance(parent_type, (astroid.ClassDef, astroid.Instance)):
-            return
+            return None
         if not has_known_bases(parent_type):
-            return
+            return None
 
         # Determine what method on the parent this index will use
         # The parent of this node will be a Subscript, and the parent of that
@@ -948,20 +1041,20 @@ accessed. Python regular expressions are accepted.'}
         try:
             methods = dunder_lookup.lookup(parent_type, methodname)
             if methods is astroid.YES:
-                return
+                return None
             itemmethod = methods[0]
         except (exceptions.NotFoundError,
                 exceptions.AttributeInferenceError,
                 IndexError):
-            return
+            return None
         if not isinstance(itemmethod, astroid.FunctionDef):
-            return
+            return None
         if itemmethod.root().name != BUILTINS:
-            return
+            return None
         if not itemmethod.parent:
-            return
+            return None
         if itemmethod.parent.name not in SEQUENCE_TYPES:
-            return
+            return None
 
         # For ExtSlice objects coming from visit_extslice, no further
         # inference is necessary, since if we got this far the ExtSlice
@@ -971,18 +1064,18 @@ accessed. Python regular expressions are accepted.'}
         else:
             index_type = safe_infer(node)
         if index_type is None or index_type is astroid.YES:
-            return
+            return None
         # Constants must be of type int
         if isinstance(index_type, astroid.Const):
             if isinstance(index_type.value, int):
-                return
+                return None
         # Instance values must be int, slice, or have an __index__ method
         elif isinstance(index_type, astroid.Instance):
             if index_type.pytype() in (BUILTINS + '.int', BUILTINS + '.slice'):
-                return
+                return None
             try:
                 index_type.getattr('__index__')
-                return
+                return None
             except exceptions.NotFoundError:
                 pass
         elif isinstance(index_type, astroid.Slice):
@@ -993,6 +1086,7 @@ accessed. Python regular expressions are accepted.'}
 
         # Anything else is an error
         self.add_message('invalid-sequence-index', node=node)
+        return None
 
     @check_messages('invalid-slice-index')
     def visit_slice(self, node):
